package ACL2::Proceso;

#
#    Clase Proceso: clase base abstracta de todo proceso de demostración
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use vars;
use utf8;

use ACL2::Localizacion;
use ACL2::Tags;
our $lh = ACL2::Localizacion->get_handle();

=head1 NAME

 ACL2::Proceso - clase base abstracta de todo proceso de demostraciE<oacute>n

=head1 FUNCTIONS

 Las funciones que aporta este mE<oacute>dulo son:

=head2 new

Uso: Constructor.

Responsabilidades: Proporciona un constructor por defecto para las
subclases.

ParE<aacute>metros: hash con el que construir la instancia.

Devuelve: objeto de la subclase correcta de ACL2::Proceso.

Fallos: croak si se intenta crear un objeto que no sea una subclase
estricta de ACL2::Proceso.

=cut

sub new
{
    my ($clase, $rh_args) = @_;

    my $ref = bless $rh_args, $clase;

    croak $lh->maketext("La clase [_1] es abstracta", __PACKAGE__) 
	if $clase eq __PACKAGE__;
    croak $lh->maketext("La clase [_1] no es subclase de [_2]", $clase, __PACKAGE__)
	unless $ref->isa(__PACKAGE__);

    return $ref;
}


=head2 escribir_xml_cabecera

Uso: ME<eacute>todo pE<uacute>blico abstracto.

Responsabilidad: Escribe la parte inicial de la salida XML del proceso.

ParE<aacute>metros: writer - XML::Writer destino de la salida.

Devuelve: Nada.

=cut

sub escribir_xml_cabecera {
    die $lh->maketext("Invocado método abstracto [_1]", (caller(1))[3]);
}


=head2 escribir_xml_final

Uso: ME<eacute>todo pE<uacute>blico abstracto.

Responsabilidad: Escribe la parte final de la salida XML del proceso.

ParE<aacute>metros: writer - XML::Writer destino de la salida.

Devuelve: Nada.

=cut

sub escribir_xml_final {
    die $lh->maketext("Invocado método abstracto [_1]", (caller(1))[3]);
}


#
# MÉTODOS ESTÁTICOS
#

=head2 obtener_reglas

Uso: ME<eacute>todo estE<aacute>tico pE<uacute>blico.

Responsabilidad: Devuelve una lista con referencias al hash con los
datos de cada regla usada en el paso dado (tipo y nombre)

ParE<aacute>metros: cad: cadena a procesar.

Devuelve: referencia a lista con referencias a hash, con campos 'tipo'
(tipo de regla) y 'nombre' (nombre de la regla).

=cut

sub obtener_reglas
{ 
    my ($paquete, $cad) = @_; 

    my @palabras = split /
	[\s\,]|
	(?-x:(primitive type reasoning))|
	(?-x:(trivial observations))|
	(?-x:(linear arithmetic))|
	(?-x:(case analysis))
	/x, $cad;

    # Tipo por defecto y lista de resultados
    my ($tipo,$rv_res) = (":DEFINITION",[]);

    for (@palabras) {
	# Nos saltamos los vacíos
	!defined($_) && next;

	# Clases de reglas
	/:compound-recognizer|:congruence|:definition|:elim|:equivalence|
	    :executable-counterpart|
	    :forward-chaining|:free-variable|:generalize|:induction|:linear|
	    :meta|:refinement|:rewrite|:type-prescription|:type-set-inverter|
	    :well-founded-relation|:definition/xi 
	    && do { $tipo = uc $&; next;  };
	
	# Procedimientos que no son reglas
	/primitive type reasoning/i && do { 	    
	    push @$rv_res, {"type" => ":FAKE-RUNE-FOR-TYPE-SET", "name" => "NIL"};
	    next; 
	};
	/trivial observations/i && do {
	    push @$rv_res, {"type" => $_, "name" => "NIL"};
	    next;
	};
	/case analysis/i && do {
	    push @$rv_res, {"type" => $_, "name" => "NIL"};
	    next;
	};
	/linear arithmetic/i && do {
	    push @$rv_res, {"type" => ":FAKE-RUNE-FOR-TYPE-SET-ARITHMETIC", "name" => "NIL"};
	    next;	    
	};
	
	# Nombres de regla (por defecto, :definitions)
	/[A-Z0-9-+*\/=]+/ && do { 
	    push @$rv_res, {"type" => uc($tipo), "name" => $_}; 
	    next;
	};
	
    }

    return $rv_res;
}


=head2 volcar_reglas

Uso: ME<eacute>todo estE<aacute>tico pE<uacute>blico.

Responsabilidades: Vuelca las reglas en el documento XML.

ParE<aacute>metros:

=over 2

=item *

writer: objeto XML::Writer destino.

=item *

rv_reglas: Referencia a vector con referencias a hash de campos "tipo" y "enunc".

=back

Devuelve: nada.

=cut

sub volcar_reglas
{
    my ($paquete, $writer, $rv_reglas) = @_;

    $writer->startTag(ACL2::Tags::RULES_TAG);
    foreach (@$rv_reglas) {
	$writer->emptyTag(ACL2::Tags::RULE_TAG, 
			  "name" => $_->{name},
			  "type" => $_->{type});
    }
    $writer->endTag(ACL2::Tags::RULES_TAG);
}


=head2 obtener_reemplazos

Uso: ME<eacute>todo estE<aacute>tico pE<uacute>blico.

Responsabilidades: Obtiene los reemplazos de tE<eacute>rminos usados.

ParE<aacute>metros: listaR: Cadena con lista de reemplazos en lenguaje
natural.

Devuelve: Devuelve una referencia a un hash con claves "origen" y
"destino" de cada reemplazo.

=cut

sub obtener_reemplazos {
    my ($paquete, $listaR) = @_;
    my @vsReemp = split / and |, |\./, $listaR;

    my %res;
    foreach (@vsReemp) {
	my @reemp = split / by /;
	$res{$reemp[0]} = $reemp[1];
    }

    return \%res;
}

1;
